/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp org.deepin.dde.GrandSearchDaemon.xml -c DaemonGrandSearchInterface -p daemongrandsearchinterface
 *
 * qdbusxml2cpp is Copyright (C) 2020 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#include "daemongrandsearchinterface.h"

/*
 * Implementation of interface class DaemonGrandSearchInterface
 */

using namespace GrandSearch;

DaemonGrandSearchInterface::DaemonGrandSearchInterface(QObject *parent)
    : QDBusAbstractInterface(staticServiceName(), staticObjectPath(), staticInterfaceName(), QDBusConnection::sessionBus(), parent)
{
}

DaemonGrandSearchInterface::~DaemonGrandSearchInterface()
{
}

