/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp com.deepin.dde.GrandSearch.xml -c GrandSearchInterface -p grandsearchinterface
 *
 * qdbusxml2cpp is Copyright (C) 2017 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#include "grandsearchinterface.h"

using namespace GrandSearch;

/*
 * Implementation of interface class GrandSearchInterface
 */

GrandSearchInterface::GrandSearchInterface(QObject *parent)
    : QDBusAbstractInterface(staticServiceName(), staticObjectPath(), staticInterfaceName(), QDBusConnection::sessionBus(), parent)
{
}

GrandSearchInterface::~GrandSearchInterface()
{
}

