/****************************************************************************
** Meta object code from reading C++ file 'vectorindex.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "vectorindex.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'vectorindex.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN37OrgDeepinAiDaemonVectorIndexInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgDeepinAiDaemonVectorIndexInterface::qt_create_metaobjectdata<qt_meta_tag_ZN37OrgDeepinAiDaemonVectorIndexInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgDeepinAiDaemonVectorIndexInterface",
        "IndexDeleted",
        "",
        "appID",
        "files",
        "IndexStatus",
        "status",
        "Create",
        "QDBusPendingReply<bool>",
        "Delete",
        "DocFiles",
        "QDBusPendingReply<QStringList>",
        "Enable",
        "Search",
        "QDBusPendingReply<QString>",
        "query",
        "topK",
        "getAutoIndexStatus",
        "setAutoIndex",
        "QDBusPendingReply<>",
        "on"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'IndexDeleted'
        QtMocHelpers::SignalData<void(const QString &, const QStringList &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QStringList, 4 },
        }}),
        // Signal 'IndexStatus'
        QtMocHelpers::SignalData<void(const QString &, const QStringList &, int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QStringList, 4 }, { QMetaType::Int, 6 },
        }}),
        // Slot 'Create'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>(const QString &, const QStringList &)>(7, 2, QMC::AccessPublic, 0x80000000 | 8, {{
            { QMetaType::QString, 3 }, { QMetaType::QStringList, 4 },
        }}),
        // Slot 'Delete'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>(const QString &, const QStringList &)>(9, 2, QMC::AccessPublic, 0x80000000 | 8, {{
            { QMetaType::QString, 3 }, { QMetaType::QStringList, 4 },
        }}),
        // Slot 'DocFiles'
        QtMocHelpers::SlotData<QDBusPendingReply<QStringList>(const QString &)>(10, 2, QMC::AccessPublic, 0x80000000 | 11, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'Enable'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>()>(12, 2, QMC::AccessPublic, 0x80000000 | 8),
        // Slot 'Search'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>(const QString &, const QString &, int)>(13, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 15 }, { QMetaType::Int, 16 },
        }}),
        // Slot 'getAutoIndexStatus'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>(const QString &)>(17, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'setAutoIndex'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &, bool)>(18, 2, QMC::AccessPublic, 0x80000000 | 19, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 20 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgDeepinAiDaemonVectorIndexInterface, qt_meta_tag_ZN37OrgDeepinAiDaemonVectorIndexInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OrgDeepinAiDaemonVectorIndexInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN37OrgDeepinAiDaemonVectorIndexInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN37OrgDeepinAiDaemonVectorIndexInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN37OrgDeepinAiDaemonVectorIndexInterfaceE_t>.metaTypes,
    nullptr
} };

void OrgDeepinAiDaemonVectorIndexInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgDeepinAiDaemonVectorIndexInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->IndexDeleted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 1: _t->IndexStatus((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 2: { QDBusPendingReply<bool> _r = _t->Create((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDBusPendingReply<bool> _r = _t->Delete((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDBusPendingReply<QStringList> _r = _t->DocFiles((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QStringList>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusPendingReply<bool> _r = _t->Enable();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<QString> _r = _t->Search((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<QString> _r = _t->getAutoIndexStatus((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QDBusPendingReply<> _r = _t->setAutoIndex((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinAiDaemonVectorIndexInterface::*)(const QString & , const QStringList & )>(_a, &OrgDeepinAiDaemonVectorIndexInterface::IndexDeleted, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinAiDaemonVectorIndexInterface::*)(const QString & , const QStringList & , int )>(_a, &OrgDeepinAiDaemonVectorIndexInterface::IndexStatus, 1))
            return;
    }
}

const QMetaObject *OrgDeepinAiDaemonVectorIndexInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgDeepinAiDaemonVectorIndexInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN37OrgDeepinAiDaemonVectorIndexInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgDeepinAiDaemonVectorIndexInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void OrgDeepinAiDaemonVectorIndexInterface::IndexDeleted(const QString & _t1, const QStringList & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void OrgDeepinAiDaemonVectorIndexInterface::IndexStatus(const QString & _t1, const QStringList & _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
