/****************************************************************************
** Meta object code from reading C++ file 'daemongrandsearchinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../3rdparty/interfaces/daemongrandsearchinterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'daemongrandsearchinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11GrandSearch26DaemonGrandSearchInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto GrandSearch::DaemonGrandSearchInterface::qt_create_metaobjectdata<qt_meta_tag_ZN11GrandSearch26DaemonGrandSearchInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GrandSearch::DaemonGrandSearchInterface",
        "Matched",
        "",
        "session",
        "SearchCompleted",
        "Configuration",
        "QDBusPendingReply<QVariantMap>",
        "Configure",
        "QDBusPendingReply<bool>",
        "QVariantMap",
        "in0",
        "FeedBackStrategy",
        "KeepAlive",
        "MatchedBuffer",
        "QDBusPendingReply<QByteArray>",
        "MatchedResults",
        "OpenWithPlugin",
        "searcher",
        "item",
        "Search",
        "key",
        "SetFeedBackStrategy",
        "Terminate",
        "QDBusPendingReply<>"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'Matched'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'SearchCompleted'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'Configuration'
        QtMocHelpers::SlotData<QDBusPendingReply<QVariantMap>()>(5, 2, QMC::AccessPublic, 0x80000000 | 6),
        // Slot 'Configure'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>(const QVariantMap &)>(7, 2, QMC::AccessPublic, 0x80000000 | 8, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'FeedBackStrategy'
        QtMocHelpers::SlotData<QDBusPendingReply<QVariantMap>()>(11, 2, QMC::AccessPublic, 0x80000000 | 6),
        // Slot 'KeepAlive'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>(const QString &)>(12, 2, QMC::AccessPublic, 0x80000000 | 8, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'MatchedBuffer'
        QtMocHelpers::SlotData<QDBusPendingReply<QByteArray>(const QString &)>(13, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'MatchedResults'
        QtMocHelpers::SlotData<QDBusPendingReply<QByteArray>(const QString &)>(15, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'OpenWithPlugin'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>(const QString &, const QString &)>(16, 2, QMC::AccessPublic, 0x80000000 | 8, {{
            { QMetaType::QString, 17 }, { QMetaType::QString, 18 },
        }}),
        // Slot 'Search'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>(const QString &, const QString &)>(19, 2, QMC::AccessPublic, 0x80000000 | 8, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 20 },
        }}),
        // Slot 'SetFeedBackStrategy'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>(const QVariantMap &)>(21, 2, QMC::AccessPublic, 0x80000000 | 8, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'Terminate'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(22, 2, QMC::AccessPublic, 0x80000000 | 23),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DaemonGrandSearchInterface, qt_meta_tag_ZN11GrandSearch26DaemonGrandSearchInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GrandSearch::DaemonGrandSearchInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11GrandSearch26DaemonGrandSearchInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11GrandSearch26DaemonGrandSearchInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11GrandSearch26DaemonGrandSearchInterfaceE_t>.metaTypes,
    nullptr
} };

void GrandSearch::DaemonGrandSearchInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DaemonGrandSearchInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->Matched((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->SearchCompleted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: { QDBusPendingReply<QVariantMap> _r = _t->Configuration();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QVariantMap>*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDBusPendingReply<bool> _r = _t->Configure((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDBusPendingReply<QVariantMap> _r = _t->FeedBackStrategy();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QVariantMap>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusPendingReply<bool> _r = _t->KeepAlive((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<QByteArray> _r = _t->MatchedBuffer((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QByteArray>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<QByteArray> _r = _t->MatchedResults((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QByteArray>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QDBusPendingReply<bool> _r = _t->OpenWithPlugin((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDBusPendingReply<bool> _r = _t->Search((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 10: { QDBusPendingReply<bool> _r = _t->SetFeedBackStrategy((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 11: { QDBusPendingReply<> _r = _t->Terminate();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DaemonGrandSearchInterface::*)(const QString & )>(_a, &DaemonGrandSearchInterface::Matched, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DaemonGrandSearchInterface::*)(const QString & )>(_a, &DaemonGrandSearchInterface::SearchCompleted, 1))
            return;
    }
}

const QMetaObject *GrandSearch::DaemonGrandSearchInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GrandSearch::DaemonGrandSearchInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11GrandSearch26DaemonGrandSearchInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int GrandSearch::DaemonGrandSearchInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void GrandSearch::DaemonGrandSearchInterface::Matched(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void GrandSearch::DaemonGrandSearchInterface::SearchCompleted(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
