/****************************************************************************
** Meta object code from reading C++ file 'matchwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../gui/exhibition/matchresult/matchwidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'matchwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11GrandSearch11MatchWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto GrandSearch::MatchWidget::qt_create_metaobjectdata<qt_meta_tag_ZN11GrandSearch11MatchWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GrandSearch::MatchWidget",
        "sigCurrentItemChanged",
        "",
        "searchGroupName",
        "MatchedItem",
        "item",
        "sigShowNoContent",
        "noContent",
        "sigCloseWindow",
        "appendMatchedData",
        "MatchedItemMap",
        "matchedData",
        "clearMatchedData",
        "onSearchCompleted",
        "onShowMore",
        "selectNextItem",
        "selectPreviousItem",
        "handleItem",
        "onPreviewStateChanged",
        "preview",
        "onSelectItemByMouse"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'sigCurrentItemChanged'
        QtMocHelpers::SignalData<void(const QString &, const MatchedItem &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'sigShowNoContent'
        QtMocHelpers::SignalData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Signal 'sigCloseWindow'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'appendMatchedData'
        QtMocHelpers::SlotData<void(const MatchedItemMap &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'clearMatchedData'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onSearchCompleted'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onShowMore'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectNextItem'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectPreviousItem'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleItem'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onPreviewStateChanged'
        QtMocHelpers::SlotData<void(const bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 19 },
        }}),
        // Slot 'onSelectItemByMouse'
        QtMocHelpers::SlotData<void(const MatchedItem &)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MatchWidget, qt_meta_tag_ZN11GrandSearch11MatchWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GrandSearch::MatchWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<Dtk::Widget::DWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11GrandSearch11MatchWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11GrandSearch11MatchWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11GrandSearch11MatchWidgetE_t>.metaTypes,
    nullptr
} };

void GrandSearch::MatchWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MatchWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sigCurrentItemChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<MatchedItem>>(_a[2]))); break;
        case 1: _t->sigShowNoContent((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->sigCloseWindow(); break;
        case 3: _t->appendMatchedData((*reinterpret_cast<std::add_pointer_t<MatchedItemMap>>(_a[1]))); break;
        case 4: _t->clearMatchedData(); break;
        case 5: _t->onSearchCompleted(); break;
        case 6: _t->onShowMore(); break;
        case 7: _t->selectNextItem(); break;
        case 8: _t->selectPreviousItem(); break;
        case 9: _t->handleItem(); break;
        case 10: _t->onPreviewStateChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->onSelectItemByMouse((*reinterpret_cast<std::add_pointer_t<MatchedItem>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MatchWidget::*)(const QString & , const MatchedItem & )>(_a, &MatchWidget::sigCurrentItemChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MatchWidget::*)(bool )>(_a, &MatchWidget::sigShowNoContent, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MatchWidget::*)()>(_a, &MatchWidget::sigCloseWindow, 2))
            return;
    }
}

const QMetaObject *GrandSearch::MatchWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GrandSearch::MatchWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11GrandSearch11MatchWidgetE_t>.strings))
        return static_cast<void*>(this);
    return Dtk::Widget::DWidget::qt_metacast(_clname);
}

int GrandSearch::MatchWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Dtk::Widget::DWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void GrandSearch::MatchWidget::sigCurrentItemChanged(const QString & _t1, const MatchedItem & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void GrandSearch::MatchWidget::sigShowNoContent(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void GrandSearch::MatchWidget::sigCloseWindow()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
