/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp com.deepin.dde.GrandSearch.xml -c GrandSearchInterface -p grandsearchinterface
 *
 * qdbusxml2cpp is Copyright (C) 2017 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef GRANDSEARCHINTERFACE_H
#define GRANDSEARCHINTERFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

namespace GrandSearch {

/*
 * Proxy class for interface com.deepin.dde.GrandSearch
 */
class GrandSearchInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "com.deepin.dde.GrandSearch"; }
    static inline const char *staticServiceName()
    { return "com.deepin.dde.GrandSearch"; }
    static inline const char *staticObjectPath()
    { return "/com/deepin/dde/GrandSearch"; }


public:
    explicit GrandSearchInterface(QObject *parent = nullptr);

    ~GrandSearchInterface();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<bool> IsVisible()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("IsVisible"), argumentList);
    }

    inline QDBusPendingReply<> SetVisible(bool visible)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(visible);
        return asyncCallWithArgumentList(QStringLiteral("SetVisible"), argumentList);
    }

Q_SIGNALS: // SIGNALS
    void VisibleChanged(bool vidible);
};

namespace com {
  namespace deepin {
    namespace dde {
      typedef GrandSearchInterface GrandSearchIfs;
    }
  }
}

}
#endif
