/****************************************************************************
** Meta object code from reading C++ file 'anything_interface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../3rdparty/interfaces/anything_interface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'anything_interface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN26ComDeepinAnythingInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto ComDeepinAnythingInterface::qt_create_metaobjectdata<qt_meta_tag_ZN26ComDeepinAnythingInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ComDeepinAnythingInterface",
        "addPathFinished",
        "",
        "path",
        "success",
        "addPath",
        "QDBusPendingReply<bool>",
        "allPath",
        "QDBusPendingReply<QStringList>",
        "cacheDir",
        "QDBusPendingReply<QString>",
        "cancelBuild",
        "hasLFT",
        "hasLFTSubdirectories",
        "insertFileToLFTBuf",
        "filePath",
        "lftBuinding",
        "parallelsearch",
        "QDBusPendingReply<QStringList,uint,uint>",
        "startOffset",
        "endOffset",
        "keyword",
        "rules",
        "QDBusReply<QStringList>",
        "uint&",
        "startOffset_",
        "endOffset_",
        "quit",
        "QDBusPendingReply<>",
        "refresh",
        "serialUriFilter",
        "removeFileFromLFTBuf",
        "removePath",
        "renameFileOfLFTBuf",
        "fromFilePath",
        "toFilePath",
        "search",
        "maxCount",
        "icase",
        "useRegExp",
        "setCodecNameForLocale",
        "QDBusPendingReply<QByteArray>",
        "name",
        "sync",
        "mountPoint",
        "autoIndexExternal",
        "autoIndexInternal",
        "logLevel"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'addPathFinished'
        QtMocHelpers::SignalData<void(const QString &, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 4 },
        }}),
        // Slot 'addPath'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>(const QString &)>(5, 2, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'allPath'
        QtMocHelpers::SlotData<QDBusPendingReply<QStringList>()>(7, 2, QMC::AccessPublic, 0x80000000 | 8),
        // Slot 'cacheDir'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>()>(9, 2, QMC::AccessPublic, 0x80000000 | 10),
        // Slot 'cancelBuild'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>(const QString &)>(11, 2, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'hasLFT'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>(const QString &)>(12, 2, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'hasLFTSubdirectories'
        QtMocHelpers::SlotData<QDBusPendingReply<QStringList>(const QString &)>(13, 2, QMC::AccessPublic, 0x80000000 | 8, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'insertFileToLFTBuf'
        QtMocHelpers::SlotData<QDBusPendingReply<QStringList>(const QByteArray &)>(14, 2, QMC::AccessPublic, 0x80000000 | 8, {{
            { QMetaType::QByteArray, 15 },
        }}),
        // Slot 'lftBuinding'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>(const QString &)>(16, 2, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'parallelsearch'
        QtMocHelpers::SlotData<QDBusPendingReply<QStringList,uint,uint>(const QString &, uint, uint, const QString &, const QStringList &)>(17, 2, QMC::AccessPublic, 0x80000000 | 18, {{
            { QMetaType::QString, 3 }, { QMetaType::UInt, 19 }, { QMetaType::UInt, 20 }, { QMetaType::QString, 21 },
            { QMetaType::QStringList, 22 },
        }}),
        // Slot 'parallelsearch'
        QtMocHelpers::SlotData<QDBusReply<QStringList>(const QString &, uint, uint, const QString &, const QStringList &, uint &, uint &)>(17, 2, QMC::AccessPublic, 0x80000000 | 23, {{
            { QMetaType::QString, 3 }, { QMetaType::UInt, 19 }, { QMetaType::UInt, 20 }, { QMetaType::QString, 21 },
            { QMetaType::QStringList, 22 }, { 0x80000000 | 24, 25 }, { 0x80000000 | 24, 26 },
        }}),
        // Slot 'parallelsearch'
        QtMocHelpers::SlotData<QDBusPendingReply<QStringList>(const QString &, const QString &, const QStringList &)>(17, 2, QMC::AccessPublic, 0x80000000 | 8, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 21 }, { QMetaType::QStringList, 22 },
        }}),
        // Slot 'quit'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(27, 2, QMC::AccessPublic, 0x80000000 | 28),
        // Slot 'refresh'
        QtMocHelpers::SlotData<QDBusPendingReply<QStringList>(const QByteArray &)>(29, 2, QMC::AccessPublic, 0x80000000 | 8, {{
            { QMetaType::QByteArray, 30 },
        }}),
        // Slot 'removeFileFromLFTBuf'
        QtMocHelpers::SlotData<QDBusPendingReply<QStringList>(const QByteArray &)>(31, 2, QMC::AccessPublic, 0x80000000 | 8, {{
            { QMetaType::QByteArray, 15 },
        }}),
        // Slot 'removePath'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>(const QString &)>(32, 2, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'renameFileOfLFTBuf'
        QtMocHelpers::SlotData<QDBusPendingReply<QStringList>(const QByteArray &, const QByteArray &)>(33, 2, QMC::AccessPublic, 0x80000000 | 8, {{
            { QMetaType::QByteArray, 34 }, { QMetaType::QByteArray, 35 },
        }}),
        // Slot 'search'
        QtMocHelpers::SlotData<QDBusPendingReply<QStringList,uint,uint>(int, qlonglong, uint, uint, const QString &, const QString &, bool)>(36, 2, QMC::AccessPublic, 0x80000000 | 18, {{
            { QMetaType::Int, 37 }, { QMetaType::LongLong, 38 }, { QMetaType::UInt, 19 }, { QMetaType::UInt, 20 },
            { QMetaType::QString, 3 }, { QMetaType::QString, 21 }, { QMetaType::Bool, 39 },
        }}),
        // Slot 'search'
        QtMocHelpers::SlotData<QDBusReply<QStringList>(int, qlonglong, uint, uint, const QString &, const QString &, bool, uint &, uint &)>(36, 2, QMC::AccessPublic, 0x80000000 | 23, {{
            { QMetaType::Int, 37 }, { QMetaType::LongLong, 38 }, { QMetaType::UInt, 19 }, { QMetaType::UInt, 20 },
            { QMetaType::QString, 3 }, { QMetaType::QString, 21 }, { QMetaType::Bool, 39 }, { 0x80000000 | 24, 25 },
            { 0x80000000 | 24, 26 },
        }}),
        // Slot 'search'
        QtMocHelpers::SlotData<QDBusPendingReply<QStringList>(const QString &, const QString &, bool)>(36, 2, QMC::AccessPublic, 0x80000000 | 8, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 21 }, { QMetaType::Bool, 39 },
        }}),
        // Slot 'setCodecNameForLocale'
        QtMocHelpers::SlotData<QDBusPendingReply<QByteArray>(const QByteArray &)>(40, 2, QMC::AccessPublic, 0x80000000 | 41, {{
            { QMetaType::QByteArray, 42 },
        }}),
        // Slot 'sync'
        QtMocHelpers::SlotData<QDBusPendingReply<QStringList>(const QString &)>(43, 2, QMC::AccessPublic, 0x80000000 | 8, {{
            { QMetaType::QString, 44 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'autoIndexExternal'
        QtMocHelpers::PropertyData<bool>(45, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'autoIndexInternal'
        QtMocHelpers::PropertyData<bool>(46, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'logLevel'
        QtMocHelpers::PropertyData<int>(47, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ComDeepinAnythingInterface, qt_meta_tag_ZN26ComDeepinAnythingInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ComDeepinAnythingInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26ComDeepinAnythingInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26ComDeepinAnythingInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26ComDeepinAnythingInterfaceE_t>.metaTypes,
    nullptr
} };

void ComDeepinAnythingInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ComDeepinAnythingInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->addPathFinished((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 1: { QDBusPendingReply<bool> _r = _t->addPath((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 2: { QDBusPendingReply<QStringList> _r = _t->allPath();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QStringList>*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDBusPendingReply<QString> _r = _t->cacheDir();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDBusPendingReply<bool> _r = _t->cancelBuild((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusPendingReply<bool> _r = _t->hasLFT((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<QStringList> _r = _t->hasLFTSubdirectories((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QStringList>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<QStringList> _r = _t->insertFileToLFTBuf((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QStringList>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QDBusPendingReply<bool> _r = _t->lftBuinding((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDBusPendingReply<QStringList,uint,uint> _r = _t->parallelsearch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[5])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QStringList,uint,uint>*>(_a[0]) = std::move(_r); }  break;
        case 10: { QDBusReply<QStringList> _r = _t->parallelsearch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<uint&>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<uint&>>(_a[7])));
            if (_a[0]) *reinterpret_cast<QDBusReply<QStringList>*>(_a[0]) = std::move(_r); }  break;
        case 11: { QDBusPendingReply<QStringList> _r = _t->parallelsearch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QStringList>*>(_a[0]) = std::move(_r); }  break;
        case 12: { QDBusPendingReply<> _r = _t->quit();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 13: { QDBusPendingReply<QStringList> _r = _t->refresh((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QStringList>*>(_a[0]) = std::move(_r); }  break;
        case 14: { QDBusPendingReply<QStringList> _r = _t->removeFileFromLFTBuf((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QStringList>*>(_a[0]) = std::move(_r); }  break;
        case 15: { QDBusPendingReply<bool> _r = _t->removePath((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 16: { QDBusPendingReply<QStringList> _r = _t->renameFileOfLFTBuf((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QStringList>*>(_a[0]) = std::move(_r); }  break;
        case 17: { QDBusPendingReply<QStringList,uint,uint> _r = _t->search((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[7])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QStringList,uint,uint>*>(_a[0]) = std::move(_r); }  break;
        case 18: { QDBusReply<QStringList> _r = _t->search((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<uint&>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<uint&>>(_a[9])));
            if (_a[0]) *reinterpret_cast<QDBusReply<QStringList>*>(_a[0]) = std::move(_r); }  break;
        case 19: { QDBusPendingReply<QStringList> _r = _t->search((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QStringList>*>(_a[0]) = std::move(_r); }  break;
        case 20: { QDBusPendingReply<QByteArray> _r = _t->setCodecNameForLocale((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QByteArray>*>(_a[0]) = std::move(_r); }  break;
        case 21: { QDBusPendingReply<QStringList> _r = _t->sync((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QStringList>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ComDeepinAnythingInterface::*)(const QString & , bool )>(_a, &ComDeepinAnythingInterface::addPathFinished, 0))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->autoIndexExternal(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->autoIndexInternal(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->logLevel(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setAutoIndexExternal(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setAutoIndexInternal(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setLogLevel(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *ComDeepinAnythingInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ComDeepinAnythingInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26ComDeepinAnythingInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int ComDeepinAnythingInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 22;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void ComDeepinAnythingInterface::addPathFinished(const QString & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
QT_WARNING_POP
