/****************************************************************************
** Meta object code from reading C++ file 'grandsearchinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../dbusservice/grandsearchinterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'grandsearchinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11GrandSearch20GrandSearchInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto GrandSearch::GrandSearchInterface::qt_create_metaobjectdata<qt_meta_tag_ZN11GrandSearch20GrandSearchInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GrandSearch::GrandSearchInterface",
        "D-Bus Interface",
        "org.deepin.dde.GrandSearchDaemon",
        "Matched",
        "",
        "session",
        "SearchCompleted",
        "Search",
        "key",
        "Terminate",
        "MatchedResults",
        "MatchedBuffer",
        "OpenWithPlugin",
        "searcher",
        "item",
        "KeepAlive"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'Matched'
        QtMocHelpers::SignalData<void(const QString &)>(3, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'SearchCompleted'
        QtMocHelpers::SignalData<void(const QString &)>(6, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'Search'
        QtMocHelpers::SlotData<bool(const QString &, const QString &)>(7, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool, {{
            { QMetaType::QString, 5 }, { QMetaType::QString, 8 },
        }}),
        // Slot 'Terminate'
        QtMocHelpers::SlotData<void()>(9, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'MatchedResults'
        QtMocHelpers::SlotData<QByteArray(const QString &)>(10, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QByteArray, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'MatchedBuffer'
        QtMocHelpers::SlotData<QByteArray(const QString &)>(11, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QByteArray, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'OpenWithPlugin'
        QtMocHelpers::SlotData<bool(const QString &, const QString &)>(12, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool, {{
            { QMetaType::QString, 13 }, { QMetaType::QString, 14 },
        }}),
        // Slot 'KeepAlive'
        QtMocHelpers::SlotData<bool(const QString &)>(15, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool, {{
            { QMetaType::QString, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<GrandSearchInterface, qt_meta_tag_ZN11GrandSearch20GrandSearchInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject GrandSearch::GrandSearchInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusService::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11GrandSearch20GrandSearchInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11GrandSearch20GrandSearchInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11GrandSearch20GrandSearchInterfaceE_t>.metaTypes,
    nullptr
} };

void GrandSearch::GrandSearchInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<GrandSearchInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->Matched((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->SearchCompleted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: { bool _r = _t->Search((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 3: _t->Terminate(); break;
        case 4: { QByteArray _r = _t->MatchedResults((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QByteArray*>(_a[0]) = std::move(_r); }  break;
        case 5: { QByteArray _r = _t->MatchedBuffer((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QByteArray*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->OpenWithPlugin((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 7: { bool _r = _t->KeepAlive((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (GrandSearchInterface::*)(const QString & )>(_a, &GrandSearchInterface::Matched, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (GrandSearchInterface::*)(const QString & )>(_a, &GrandSearchInterface::SearchCompleted, 1))
            return;
    }
}

const QMetaObject *GrandSearch::GrandSearchInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GrandSearch::GrandSearchInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11GrandSearch20GrandSearchInterfaceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    return QDBusService::qt_metacast(_clname);
}

int GrandSearch::GrandSearchInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusService::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void GrandSearch::GrandSearchInterface::Matched(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void GrandSearch::GrandSearchInterface::SearchCompleted(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
